local array = require('libraries/zArray')

local Grid = class('Grid')

function Grid:initialize(width, height, cellSize)
	self.width = width
	self.height = height
	self.cellSize = cellSize

	self.cells = {}
end

function Grid:mousepressed(x, y, button)
	local cellX = math.floor(x / self.cellSize) + 1
	local cellY = math.floor(y / self.cellSize) + 1

	local cell = array.check(self.cells, cellX, cellY)
	if cell and cell.mousepressed then
		cell:mousepressed(button)
	end

	return cell, cellX, cellY
end

function Grid:draw(xOffset, yOffset)
	local cellSize = self.cellSize

	for x in pairs(self.cells) do
		for y in pairs(self.cells[x]) do
			local drawX = xOffset + cellSize*(x-1)
			local drawY = yOffset + cellSize*(y-1)

			local cell = self.cells[x][y]

			cell:draw(drawX, drawY, cellSize)
		end
	end

	self:_drawLines(xOffset, yOffset, cellSize)
end

function Grid:insert(item, x, y)
	local oldItem = array.check(self.cells, x, y)

	array.set(self.cells, x, y, item)

	if oldItem then
		return oldItem
	end
end

function Grid:check(x, y)
	return array.check(self.cells, x, y)
end

function Grid:getPos(item)
	for x in pairs(self.cells) do
		for y in pairs(self.cells) do
			if self.cells[x][y] == item then
				return x, y
			end
		end
	end
end

function Grid:_drawLines(x, y)
	local cellSize = self.cellSize

	love.graphics.setColor(255, 255, 255, 255)
	for line=0, self.width do
		local x1 = x + line*cellSize
		local y1 = y
		local y2 = y + (cellSize*self.height)
		love.graphics.line(x1, y1, x1, y2)
	end

	for line=0, self.height do
		local x1 = x
		local x2 = x + (cellSize*self.width)
		local y1 = y + line*cellSize
		love.graphics.line(x1, y1, x2, y1)
	end
end

return Grid
