local Scene = class('Scene')

function Scene:initialize()
	self.groups = {}
end

function Scene:load(file)
	barrack.check({file}, {'string'})
	local file = 'scene/'..file
	local scene = require(file)
	scene:initialize()

	return scene
end

-- Instance Groups
function Scene:newInstance(class, args, ...)
	barrack.check({class, args}, {'middleclassClass', 'table'})

	local instance = class:new(unpack(args))
	instance._groups = {}
	instance._scene = self

	local groups = {'all', instance.class.name, ...}

	self:add(instance, unpack(groups))

	return instance
end

function Scene:destroyInstance(instance)
	barrack.check({instance}, 'middleclassInstance')

	for _, group in pairs(instance._groups) do
		self:remove(instance, group)
	end
	instance = nil
end


function Scene:add(instance, ...)
	barrack.check({instance}, {'middleclassInstance'})

	local groups = {...}
	for i=1, #groups do
		local group = groups[i]
		-- Create the group if it does not exist yet
		if not self.groups[group] then
			self.groups[group] = {}
		end

		-- Check if the instance is already in the group
		local found = false
		for j=1, #self.groups[group] do
			if self.groups[group][j] == instance then
				--print("[Warning] '"..instance.class.name.."' is already a member of '"..group.."'")
				found = true
				break
			end
		end

		-- If the instance was not in the group, add it
		if found == false then
			table.insert(self.groups[group], instance)
			table.insert(instance._groups, group)
		end
	end
end

function Scene:remove(instance, group)
	barrack.check({instance, group}, {'middleclassInstance', 'string'})

	for i=1, #self.groups[group] do
		if instance == self.groups[group][i] then
			return table.remove(self.groups[group], i)
		end
	end
	--print("[Warning] No instance of '"..instance.class.name.."' to remove from group '"..group.."'")
end

function Scene:exec(group, func, ...)
	barrack.check({group, func}, {'string', 'string'})

	local args = {...}
	local output = {}

	if self.groups[group] then
		for i=1, #self.groups[group] do
			local instance = self.groups[group][i]
			if instance[func] then
				local o = instance[func](instance, unpack(args))
				table.insert(output, o)
			else
				--print("[Warning] Instance '"..instance.class.name.."' does not have a function '"..func.."'")
			end
		end
	else
		--print("[Warning] ExecList '"..group.."' does not exist")
	end

	return output
end

return Scene
